; This crib sheet is a demo of using THM_LSP_GET_SPEC designed to get the power
; spectrum from field data, whose output can be used to identify waves.
;
; Two cases are included in this crib sheet. In case 2, the usage of
;  THM_LSP_EZFILTER is demonstrated.
;
; History:
;     2010-04-04: Created by Jianbao Tao (JBT), CU/LASP.
 
 
;===============================================================================
; Case 1.
;   From this paper:
;   O. Le Contel et al., Annales Geophysicae, Volume 27, Issue 6, 2009,
;   pp.2259-2275

   ; Some common stuff.
   sc = 'd'
   thx0 = 'th' + sc
   thx = thx0 + '_'

   date = '2008-01-29'

   timespan, date
   thm_load_state, probe=sc, /get_support

   ; Load field data.
   thm_load_scm, probe=sc, datatype='scp', coord='dsl'

   ; Get spectra from field data.
   tvar = thx + 'scp'
   prefix = thx + 'scpdsl_B' ; prefix of the resulting tplot variables
   yrange = [0.1, 70] ; yrange for the spectral plot
   fftlen = 128 ; array length for one FFT operation
   thm_lsp_get_spec, tvar, prefix=prefix, yrange = yrange, fftlen = fftlen

   ; Plot data.
   tr = time_double(date+['/02:24:00', '/02:28:00']) ; time range
   tlist = [tvar, prefix + ['xspec', 'yspec', 'zspec']]
   tplot, tlist, trange=tr

;===============================================================================
; Case 2.
;  From this paper:
;  Cully et al., Geophys. Res. Lett., 35, L17S16, doi:10.1029/2008GL033643. 
   sc = 'e'
   thx0 = 'th' + sc
   thx = thx0 + '_'

   date = '2007-11-14'

   timespan, date
   thm_load_state, probe=sc, /get_support

   ; Load field data.
   thm_efi_clean_efw, probe=sc

   ; Clip data.
   tr = time_double(date+['/03:35:07.6', '/03:35:08.2']) ; time range
   clipname = thx + 'efwdsl_clip'
   trange_clip, thx + 'efw_clean_dsl', tr[0], tr[1], $
         newname = clipname

   ; Get spectra from field data.
   tvar = clipname
   prefix = thx + 'efwdsl_clip_E' ; prefix of the resulting tplot variables
   yrange = [10, 1e4] ; yrange for the spectral plot
   fftlen = 512 ; array length for one FFT operation
   thm_lsp_get_spec, tvar, prefix=prefix, yrange = yrange, fftlen = fftlen

   ; Plot data.
   tlist = [tvar, prefix + ['xspec', 'yspec', 'zspec']]
   tplot, tlist, trange=tr

   ; Filter data with thm_lsp_ezfilter.
   nyquist = 8192. / 2
   flow = 100. / nyquist
   fhigh = 1. ; high-pass filter
   amp = 120. ; 
   nterms = 200. ;
   filter = digital_filter(flow, fhigh, amp, nterms, /double)
   tvarhp = tvar + '_hp'
   thm_lsp_ezfilter, tvar, filter, newname = tvarhp

   prefix = thx + 'efwdslhp_clip_E' ; prefix of the resulting tplot variables
   thm_lsp_get_spec, tvarhp, prefix=prefix, yrange = yrange, fftlen = fftlen
   
   ; Plot high-pass filtered data.
   tlist = [tvarhp, prefix + ['xspec', 'yspec', 'zspec']]
   tplot, tlist, trange=tr

 End
