;+
; NAME:
;     THM_LSP_EZFILTER (PROCEDURE)
;
; PURPOSE:
;     Simply wrap up the process of filtering a tplot varialbe with a filter
;     from digital_filter.
;
; CALLING SEQUENCE:
;     thm_lsp_ezfilter, tvar, filter, newname=newname
;
; ARGUMENTS:
;     tvar: (INPUT, REQUIRED) The name of a field tplot variable.
;     filter: (INPUT, REQUIRED) A filter generated by the routine
;           DIGITAL_FILTER.
;
; KEYWORDS:
;     newname: (INPUT, OPTIONAL) A name for storing the filtered data into a new
;           tplot variable.
;
; EXAMPLES:
;     See THM_CRIB_LSP_GET_SPEC which is typically located under
;     TDAS_DIR/idl/themis/examples.
;
; HISTORY:
;     2010-03-06: Created by JBT, CU/LASP.
;
;-

pro thm_lsp_ezfilter, tvar, filter, newname=newname

; Check arguments.
thm_lsp_checkarg_tvar, tvar, error, errmsg
if error ne 0 then begin
   print, 'THM_LSP_EZFILTER: ' + errmsg + ' Exiting..'
   return
endif

nf = n_elements(filter)
if (nf eq 0) or (nf mod 2 ne 1) then begin
   print, 'THM_LSP_EZFILTER: ' + $
      'A filter generated by the routine DIGITAL_FILTER must be provided. '+$
      'Exiting...'
   return
endif

get_data, tvar, data=data, dlim = dlim
tarr = data.x
ydim = size(data.y, /dim)
if ydim[1] eq 3 then begin
   ex = data.y[*,0]
   ey = data.y[*,1]
   ez = data.y[*,2]
endif
if ydim[1] eq 6 then begin
   v1 = data.y[*,0]
   v2 = data.y[*,1]
   v3 = data.y[*,2]
   v4 = data.y[*,3]
   v5 = data.y[*,4]
   v6 = data.y[*,5]
endif

if n_elements(newname) ne 1 then newname = tvar

btrange = thm_jbt_get_btrange(tvar, nb = nb, tind = tind)

nf = n_elements(filter)
for ib = 0L, nb -1L do begin
   ista = tind[ib, 0]
   iend = tind[ib, 1]
   tmptarr = tarr[ista:iend]
   dt = median(tmptarr[1:*] - tmptarr)
   if ydim[1] eq 3 then begin
      tmpex = ex[ista:iend]
      tmpey = ey[ista:iend]
      tmpez = ez[ista:iend]
      ; remove offset
      tmpex = tmpex - median(tmpex) 
      tmpey = tmpey - median(tmpey) 
      tmpez = tmpez - median(tmpez) 
      nn = n_elements(tmpex)
      if nn ge nf then tmpex = convol(tmpex, filter, /edge_w)
      if nn ge nf then tmpey = convol(tmpey, filter, /edge_w)
      if nn ge nf then tmpez = convol(tmpez, filter, /edge_w)
      ex[ista:iend] = tmpex
      ey[ista:iend] = tmpey
      ez[ista:iend] = tmpez
   endif
   if ydim[1] eq 6 then begin
      tmpv1 = v1[ista:iend]
      tmpv2 = v2[ista:iend]
      tmpv3 = v3[ista:iend]
      tmpv4 = v4[ista:iend]
      tmpv5 = v5[ista:iend]
      tmpv6 = v6[ista:iend]
      ; remove offset
      tmpv1 = tmpv1 - median(tmpv1)
      tmpv2 = tmpv2 - median(tmpv2)
      tmpv3 = tmpv3 - median(tmpv3)
      tmpv4 = tmpv4 - median(tmpv4)
      tmpv5 = tmpv5 - median(tmpv5)
      tmpv6 = tmpv6 - median(tmpv6)
      nn = n_elements(tmpv1)
      if nn ge nf then tmpv1 = convol(tmpv1, filter, /edge_w) 
      if nn ge nf then tmpv2 = convol(tmpv2, filter, /edge_w) 
      if nn ge nf then tmpv3 = convol(tmpv3, filter, /edge_w) 
      if nn ge nf then tmpv4 = convol(tmpv4, filter, /edge_w) 
      if nn ge nf then tmpv5 = convol(tmpv5, filter, /edge_w) 
      if nn ge nf then tmpv6 = convol(tmpv6, filter, /edge_w) 
      v1[ista:iend] = tmpv1
      v2[ista:iend] = tmpv2
      v3[ista:iend] = tmpv3
      v4[ista:iend] = tmpv4
      v5[ista:iend] = tmpv5
      v6[ista:iend] = tmpv6
   endif
endfor

if ydim[1] eq 3 then begin
   data.y[*,0] = ex
   data.y[*,1] = ey
   data.y[*,2] = ez
endif
if ydim[1] eq 6 then begin
   data.y[*,0] = v1
   data.y[*,1] = v2
   data.y[*,2] = v3
   data.y[*,3] = v4
   data.y[*,4] = v5
   data.y[*,5] = v6
endif

store_data, newname, data = data, dlim = dlim

end
