;+
;NAME:
; thm_sst_build_cal_files
;PURPOSE:
;
;  Automated routine to ease the creation of text-based cal-files
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2012-02-17 15:08:53 -0800 (Fri, 17 Feb 2012) $
;$LastChangedRevision: 9789 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/spacecraft/particles/SST/SST_cal_workdir/thm_sst_build_cal_files.pro $
;-


pro thm_sst_build_cal_files

  cal_file_root = '~/IDLWorkspace/themis/spacecraft/particles/SST/SST_cal_workdir/cal_files/'
  probes=['a','b','c','d','e']
  
  ;based on geometry only
  gf_nominal = 0.1
  ;dead time correction not yet completed
  deadtime = 0.0
  ;notional attentuator factor.  Attenuator factors TBD empirically(comparing delta flux from an attenuator close/open)
  attenuator_correction = 1/64.
  
  ;geant4 modeled energy boundaries
  ;deadlayers TBD empirically (looking at changing performance across time & after bias voltage steps)
  ;numbers are not strictly ascending because some of the boundaries are for SST coincidence channels.(FT,OT,FTO,FO)
  ion_open_en_low=[25,36,46,59,75,115,168,243,346,492,812,1453,6315,6485,6485,6485]  
  ion_open_en_high=[37,48,65,77,116,175,245,348,494,813,1455,6500,6485,6485,6485,6485]
  
  electron_foil_en_low=[26,36,46,58,73,113,165,241,345,495,600,600,335,481,640,710]
  electron_foil_en_high=[36,46,58,73,113,165,242,345,495,600,600,600,481,642,799,1409]
  
  ;geant modeled efficiencies.  
  ;These are applied only as a scalar divisor to each channel.
  ;Doesn't do any matrix-style decontamination/crosstalk removal
  ;However, future versions intend to use an efficiency decontamination matrix
  electron_foil_mono_channel_efficiencies=[0.35,0.49,0.52,0.59,0.73,0.77,0.77,0.75,0.43,0.16,0.05,0.05,0.34,0.60,0.60,0.25]
  ion_open_mono_channel_efficiencies=[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
    
  ;dates during which anode intercalibrations have been performed
  ;intercalibration values are used from date i to date i+1, there is no interpolation across the boundary.  It will step to the new calibration factor
  tha_anode_intercalibration_dates=['1970-01-01/00:00:00',$ ;sentinel(slightly abusing the term)
                                    '2007-07-01/00:00:00',$
                                    '2007-11-01/00:00:00',$
                                    '2007-12-12/00:00:00',$
                                    '2008-05-01/00:00:00',$
                                    '2009-05-01/00:00:00']
                                
  tha_ion_anode_intercalibrations=[[1.16,1.24,0.70,1.40],$
                                   [0.97,1.24,0.90,1.05],$
                                   [1.06,1.56,0.66,1.42],$
                                   [1.20,1.29,0.63,1.53],$
                                   [1.21,1.17,0.68,1.45],$
                                   [0.96,0.84,0.84,1.60]]
                                 
                                   
  tha_electron_anode_intercalibrations=[[0.98,0.99,0.94,1.06],$
                                        [1.01,0.99,1.08,0.95],$
                                        [0.99,0.96,1.04,1.02],$
                                        [0.95,1.01,1.08,0.98],$
                                        [0.96,1.02,0.93,1.08],$
                                        [0.94,0.99,0.96,1.13]]
                                        
  thb_anode_intercalibration_dates=['1970-01-01/00:00:00',$
                                    '2007-12-12/00:00:00',$
                                    '2009-05-01/00:00:00',$
                                    '2011-10-01/00:00:00']
   
  thb_ion_anode_intercalibrations=[[0.91,0.71,0.68,26.4],$
                                   [0.98,0.87,0.95,1.14],$
                                   [0.92,0.79,1.08,1.24],$
                                   [0.67,0.76,1.62,1.35]]
                                   
                                        
  thb_electron_anode_intercalibrations=[[0.77,1.03,1.04,1.04],$
                                        [0.88,1.13,0.95,1.07],$
                                        [0.89,1.13,0.95,1.05],$
                                        [0.89,1.13,0.95,1.05]]
      
  thc_anode_intercalibration_dates=['1970-01-01/00:00:00',$
                                    '2007-12-12/00:00:00',$
                                    '2009-05-01/00:00:00',$
                                    '2011-10-01/00:00:00']
                                        
  thc_ion_anode_intercalibrations=[[9.31,0.30,6.30,3.09],$
                                   [1.08,0.71,1.36,0.99],$
                                   [1.09,0.66,2.08,0.94],$
                                   [0.87,0.63,1.63,1.03]]
 
  thc_electron_anode_intercalibrations=[[0.79,1.12,1.02,1.14],$
                                        [0.87,1.18,0.93,1.08],$
                                        [0.83,1.14,0.94,1.11],$
                                        [0.83,1.14,0.94,1.11]]
       
  thd_anode_intercalibration_dates=['1970-01-01/00:00:00',$
                                    '2007-12-12/00:00:00',$
                                    '2009-05-01/00:00:00']
                                        
  thd_ion_anode_intercalibrations=[[0.96,0.75,4.86,0.68],$
                                   [0.99,0.77,1.00,1.28],$
                                   [0.93,0.66,1.19,1.88]]
                                   
  thd_electron_anode_intercalibrations=[[0.94,0.97,1.02,1.04],$
                                        [0.91,1.08,0.96,1.05],$
                                        [0.87,1.02,1.07,1.09]]
                                        
  the_anode_intercalibration_dates=['1970-01-01/00:00:00',$
                                    '2007-12-12/00:00:00',$
                                    '2008-05-01/00:00:00',$
                                    '2009-05-01/00:00:00']
                                   
  the_ion_anode_intercalibrations=[[1.00,1.31,0.88,0.89],$
                                   [0.95,1.13,0.93,1.04],$
                                   [0.85,1.02,0.97,1.09],$
                                   [0.79,0.95,1.09,1.24]]     
                                   
  the_electron_anode_intercalibrations=[[1.30,0.85,1.05,0.84],$
                                        [1.05,0.82,1.29,0.97],$
                                        [1.14,0.73,1.21,1.12],$
                                        [1.06,0.71,1.27,1.20]]
   
   
   
  openw,lun,cal_file_root+'tha_psif_calib_params.txt',/get_lun
  printf,lun,'#THA PSIF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(tha_anode_intercalibration_dates)-1 do begin
    s=tha_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(ion_open_en_low,2),' ')+' '+$
      strjoin(strtrim(ion_open_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(tha_ion_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(ion_open_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
      
  openw,lun,cal_file_root+'tha_psef_calib_params.txt',/get_lun
  printf,lun,'#THA PSEF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(tha_anode_intercalibration_dates)-1 do begin
    s=tha_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(electron_foil_en_low,2),' ')+' '+$
      strjoin(strtrim(electron_foil_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(tha_electron_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(electron_foil_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
  openw,lun,cal_file_root+'thb_psif_calib_params.txt',/get_lun
  printf,lun,'#THB PSIF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(thb_anode_intercalibration_dates)-1 do begin
    s=thb_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(ion_open_en_low,2),' ')+' '+$
      strjoin(strtrim(ion_open_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(thb_ion_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(ion_open_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
      
  openw,lun,cal_file_root+'thb_psef_calib_params.txt',/get_lun
  printf,lun,'#THB PSEF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(thb_anode_intercalibration_dates)-1 do begin
    s=thb_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(electron_foil_en_low,2),' ')+' '+$
      strjoin(strtrim(electron_foil_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(thb_electron_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(electron_foil_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
  openw,lun,cal_file_root+'thc_psif_calib_params.txt',/get_lun
  printf,lun,'#THC PSIF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(thc_anode_intercalibration_dates)-1 do begin
    s=thc_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(ion_open_en_low,2),' ')+' '+$
      strjoin(strtrim(ion_open_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(thc_ion_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(ion_open_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
      
  openw,lun,cal_file_root+'thc_psef_calib_params.txt',/get_lun
  printf,lun,'#THC PSEF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(thc_anode_intercalibration_dates)-1 do begin
    s=thc_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(electron_foil_en_low,2),' ')+' '+$
      strjoin(strtrim(electron_foil_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(thc_electron_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(electron_foil_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
       
  openw,lun,cal_file_root+'thd_psif_calib_params.txt',/get_lun
  printf,lun,'#THD PSIF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(thd_anode_intercalibration_dates)-1 do begin
    s=thd_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(ion_open_en_low,2),' ')+' '+$
      strjoin(strtrim(ion_open_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(thd_ion_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(ion_open_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
      
  openw,lun,cal_file_root+'thd_psef_calib_params.txt',/get_lun
  printf,lun,'#THD PSEF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(thd_anode_intercalibration_dates)-1 do begin
    s=thd_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(electron_foil_en_low,2),' ')+' '+$
      strjoin(strtrim(electron_foil_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(thd_electron_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(electron_foil_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun   
       
  openw,lun,cal_file_root+'the_psif_calib_params.txt',/get_lun
  printf,lun,'#THE PSIF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(the_anode_intercalibration_dates)-1 do begin
    s=the_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(ion_open_en_low,2),' ')+' '+$
      strjoin(strtrim(ion_open_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(the_ion_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(ion_open_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun
  
      
  openw,lun,cal_file_root+'the_psef_calib_params.txt',/get_lun
  printf,lun,'#THE PSEF CALIBRATION FILE, see thm_sst_build_cal_files.pro for detailed descriptions of each parameter' 
  for i=0,n_elements(the_anode_intercalibration_dates)-1 do begin
    s=the_anode_intercalibration_dates[i]+' '+$
      strjoin(strtrim(electron_foil_en_low,2),' ')+' '+$
      strjoin(strtrim(electron_foil_en_high,2),' ')+' '+$
      strtrim(deadtime,2)+' '+$
      strtrim(gf_nominal,2)+' '+$
      strjoin(strtrim(the_electron_anode_intercalibrations[*,i],2),' ')+' '+$
      strtrim(attenuator_correction,2)+' '+$
      strjoin(strtrim(electron_foil_mono_channel_efficiencies,2),' ')
    printf,lun,s  
  endfor
  close,lun
  free_lun,lun    
  
  dprint,'Calibration File Generation Completed',dlevel=2  ;dlevel 2 is informational message
            
end
                                                
   