; Removes trailing zeros and/or decimal from string,
; Assumes trailing spaces have already been removed.
function remove_zeros, sval

    compile_opt idl2, hidden
  
  if ~stregex(sval, '\.', /bool) then return, sval

  f = stregex(sval, '0*$',length=len)

  if stregex(sval, '\.0*$', /bool) then len++

  return, strmid(sval, 0, (strlen(sval)-len) )

end



;pro plot_part_slice2d_ranges, part_slice = part_slice, $
;                  xgrid=xgrid, ygrid=ygrid, slice_info=slice_info, $ 
;                  o_part_slice = o_part_slice, $
;                  o_xgrid=o_xgrid, o_ygrid=o_ygrid, o_slice_info=o_slice_info, $
;                  range = range, $
;                  xrange = xrange, $
;                  yrange = yrange
;
;    compile_opt idl2, hidden
;
;  if size(o_part_slice,/type) eq 0 then return
;  if size(o_xgrid,/type) eq 0 then return
;  if size(o_ygrid,/type) eq 0 then return
;  if size(o_slice_info,/type) ne 8 then return
;
;  ; Ensure z range will cover both slices
;  if ~keyword_set(range) then begin
;    slice_info.range = minmax([slice_info.range,o_slice_info.range])
;  endif
;
;  ; Ensure x,y ranges will contain both slices
;  if ~keyword_set(xrange) then begin
;    xrange = minmax([xgrid,o_xgrid])
;  endif
;  
;  if ~keyword_set(yrange) then begin
;    yrange = minmax([ygrid,o_ygrid])
;  endif
;  
;
;end



; Procedure to generate titles for plot and axes
;
pro plot_part_slice2d_getinfo, slice_info, title=title, subtitle=subtitle, $
                               xtitle=xtitle, ytitle=ytitle, ztitle=ztitle

    compile_opt idl2, hidden


  ; Plot title
  if arg_present(title) and ~keyword_set(title) then begin
    title = 'th'+strlowcase(slice_info.probe)+' ' + $
            strjoin(slice_info.dist,'/') + $
            ' ('+slice_info.rot+') ' + $
;            string(10b) + $ 
            time_string(slice_info.trange[0]) + $
            ' -> '+strmid(time_string(slice_info.trange[1]),11,8)
  endif

  ; X,Y axes labels
  xc = 1 & yc = 1
  if arg_present(xtitle) or arg_present(ytitle) then begin
    xyunits = 'km/s'
    case slice_info.rot of 
      'BV': begin
        xtitle = 'V!DB!N'
        ytitle = 'V!DV!N'
        xc = 0 & yc = 0
      end
      'BE': begin
        xtitle = 'V!DB!N'
        ytitle = 'V!DB x V!N'
        xc = 0 & yc = 0
      end
      'xy': begin
        xtitle = 'Vx'
        ytitle = 'Vy'
      end
      'xz': begin
        xtitle = 'Vx'
        ytitle = 'Vz'
      end
      'yz': begin
        xtitle = 'Vy'
        ytitle = 'Vz'
      end
      'xvel': begin
        xtitle = 'Vx'
        ytitle = 'V!DV!N'
        yc = 0
      end
      'perp': begin
        xtitle = 'V!DV perp B!N'
        ytitle = 'V!DB x V!N'
        xc = 0 & yc = 0
      end
      'perp_xy': begin
        xtitle = 'Vx!Dperp!N'
        ytitle = 'Vy!Dperp!N'
      end
      'perp_xz': begin
        xtitle = 'Vx!Dperp!N'
        ytitle = 'Vz!Dperp!N'
      end
      'perp_yz': begin
        xtitle = 'Vy!Dperp!N' 
        ytitle = 'Vz!Dperp!N'
      end
      else: begin
        xtitle = 'Vx!D?!N'
        ytitle = 'Vy!D?!N'
      end
    endcase
    
    ; add units 
    xtitle += ' ('+xyunits+')'
    ytitle += ' ('+xyunits+')'
    
    ; specify original physical coords
    if xc then xtitle += ' ('+strupcase(slice_info.coord)+')'
    if yc then ytitle += ' ('+strupcase(slice_info.coord)+')'

    ; check for rotation out of x-y plane and label
    if n_elements(slice_info.orient_m) gt 1 then begin
      xvec = [1,0,0]##slice_info.orient_m
      yvec = [0,1,0]##slice_info.orient_m
      if in_set(xvec eq [1,0,0],0) then begin
        xv = strtrim(xvec,2)
        xtitle += ' - Using ('+ $
                  remove_zeros(xv[0])+', '+ $
                  remove_zeros(xv[1])+', '+ $
                  remove_zeros(xv[2])+') as x-axis'
      endif
      if in_set(yvec eq [0,1,0],0) then begin
        yv = strtrim(yvec,2)
        ytitle += ' - Using ('+ $
                  remove_zeros(yv[0])+', '+ $
                  remove_zeros(yv[1])+', '+ $
                  remove_zeros(yv[2])+') as y-axis'
      endif
    endif
  endif
  
  
  ; Z axis label
  if arg_present(ztitle) and ~keyword_set(ztitle) then begin 
    ; convert time window to string and trim zeros
    twin = remove_zeros(strtrim(slice_info.twin,2))
    ztitle = units_string(strlowcase(slice_info.units))
  endif


  ; Label slice type
  if arg_present(type) then begin
    type = (['Geo','2D nn','2D i','3D i'])[slice_info.type]
  endif


end



; Find the sun direcion vector
function plot_part_slice2d_getsun, slice_info, fail=fail

    compile_opt idl2, hidden

  
  coord = strlowcase(slice_info.coord)
  probe = strlowcase(slice_info.probe)
  trange = slice_info.trange
  
  ; Caculation needed for dsl
  if stregex(coord, '^dsl$', /bool, /fold_case) then begin 
    dprint, 'Finding sun direction in DSL...'
    ctn = '2dslice_temp_sundir'
    
    ;Get support data
    thm_load_state, probe=probe, trange=trange + 120*[-1,1], suffix='_'+ctn, $
                    /get_support_data

    ;Transform GSE basis vectors
    store_data, ctn, data = {x:replicate(mean(trange),3), $
                             y: [ [1.,0,0], [0,1,0], [0,0,1] ] }
    thm_cotrans, ctn, probe=probe, in_coord='gse', out_coord=coord, $
                 support_suffix='_'+ctn, out_suff='_'+coord

    ;Transformed x-axis is DSL sun direction
    get_data, ctn+'_'+coord, data=ctd
    if size(ctd,/type) ne 8 then begin
      fail = 'Could not obtain coordinate transform from GSE -> DSL'+ $
             ', check for error from THM_COTRANS.'
      dprint, fail
      return, -1
    endif
    sundir = ctd.y[0,*]
    
    ;Delete temporary tplot data
    store_data, '*'+ctn+'*', /delete
  endif else begin
  
    sundir = transpose([1,0,0]) ; By definition for GSE/GSM
  
  endelse

  ;correct for rotation
  sundir = slice_info.rot_m ## sundir
  
  ;correct for user specified rotations (*new slices only)
  if n_elements(slice_info.orient_m) gt 1 then begin
   sundir = slice_info.orient_m ## sundir
  endif

  return, sundir

end



;+
;Procedure: plot_part_slice2d
;
;Purpose: Create plots for 2D particle slices.
;
;Calling Sequence:
;  Single slice
;  plot_part_slice2d, part_slice, xgrid, ygrid, slice_info, $
;
;Arguments:
; PART_SLICE: 2D array of values to plot 
; XGRID: X location of data points 
; YGRID: Y location of data points
; SLICE_INFO: structure passed out of thm_part_slice2d containing
;             information for labels and range for plotting
;
;Other Parameters:
; RANGE: Two-element array specifying zaxis range (units vary)
; LEVELS: Number of color contour levels to plot (default is 60)
; OLINES: Number of contour lines to plot (default is 0)
; PLOTSIZE: The size of the plot in device units (usually pixels)
;           (Not yet implemented for postscript).
;
;Keywords:
; LOGPLOT: Boolean indicating plot should be log-scaled (on by default)
; SUNDIR: Boolean to plot sun direction vector (on by default)
; ECIRCLE: Boolean to plot circle(s) designating min/max energy 
;          from distribution (on by default)
; PLOTAXES: Boolean to plot zero axes (on by default)
; PLOTBULK: Boolean to plot projection of bulk velocity vector on the 
;           slice plane (on by default)
; CLABELS: Boolean to annotate contour lines.
; SCALE: Numerical value to scale O_PART_SLICE by. 
; [XYZ]TICKS: Integer(s) specifying the number of ticks for each axis 
; CHARSIZE: Specifies character size of annotations (1 is normal)
; APRECISION: Integer specifying annotation precision (sig. figs.).
;             Set to zero to truncate printed values to inegers.
; ASTYLE: Integer specifying annotation style:
;         Set to 0 (default) for style to be chosen automatically. 
;         Set to 1 for decimal annotations only ('0.0123') 
;         Set to 2 for scientific notation only ('1.23e-2')
;
;Exporting keywords:
; EXPORT: String designating the path and file name of the desired file. 
;         The plot will be exported to a PNG image by default.
; EPS: Boolean indicating that the plot should be exported to 
;      encapsulated postscript.
;
;
;Created by: A. Flores
;            Based on work by Bryan Kerr and Arjun Raj

;-

pro plot_part_slice2d, part_slice, xgrid, ygrid, slice_info, $
                     ; Range options
                       range=range, xrange=xrange, yrange=yrange, $
                     ; Basic plotting options
                       title=title, ztitle=ztitle, $ 
                       xtitle=xtitle, ytitle=ytitle, $
                       xticks=xticks, yticks=yticks, zticks=zticks, $
                       charsize=charsize, plotsize=plotsize, $
                       logplot=logplot, $
                     ; Annotations
                       astyle=astyle, aprecision=aprecision, $
                     ; Contours
                       olines=olines, level=levels, nlines=nlines, clabels=clabels, $
                     ; Other plotting options
                       plotaxes=plotaxes, ecircle=ecircle, sundir=sundir, $ 
                       plotbulk=plotbulk, $
                     ; Eport
                       export=export, eps=eps, $
                       _extra=_extra

    compile_opt idl2

  ; Return if variables are not set
  if size(part_slice,/type) eq 0 then $
    fail = 'Slice data is missing (PART_SLICE), canceling plot.'
  if size(xgrid,/type) eq 0 then $
    fail = 'X-axis values are missing (XGRID), canceling plot.'
  if size(ygrid,/type) eq 0 then $
    fail = 'Y-axis values are missing (YGRID), canceling plot.'
  if size(slice_info,/type) ne 8 then $ 
    fail = 'Slice info is missing (SLICE_INFO), canceling plot.'
  if keyword_set(fail) then begin
    dprint, fail
    return
  endif


  ; Defaults
  if slice_info.type eq 2 then begin
    if size(olines,/type) eq 0 then olines = 20
  endif
  if keyword_set(nlines) and ~keyword_set(levels) then levels = nlines ;temp
  if ~keyword_set(levels) then levels=60
  if size(zticks,/type) eq 0 then zticks=11
  if size(logplot,/type) eq 0 then logplot=1b
  if size(plotaxes,/type) eq 0 then plotaxes=1b
;  if size(ecircle,/type) eq 0 then ecircle=1b ;todo: fix
  if size(plotbulk,/type) eq 0 then plotbulk=1b
;  if size(sundir,/type) eq 0 then sundir=1b ;may require STATE data
  if size(astyle,/type) eq 0 then astyle=0
  if size(aprecision,/type) eq 0 then aprecision=3
  if ~keyword_set(plotsize) then plotsize = 500.
  plotsize = 100. > plotsize ;min size


  ;Open postscript file
  if keyword_set(export) && keyword_set(eps) then begin
    popen, outputfile, encapsulated = 1, _extra=_extra
  endif


  ; Main Plot
  ; ---------

  ; X,Y,Z ranges
  if keyword_set(xrange) then begin
    xrange = minmax(xrange)
  endif else begin
    xrange = minmax(xgrid)
  endelse

  if keyword_set(yrange) then begin
    yrange = minmax(yrange)
  endif else begin
    yrange = minmax(ygrid)
  endelse

  if keyword_set(range) then begin
    range = minmax(range)    
  endif else begin
    ; The default range is the nonnegative, nonzero 
    ; min/max of the pre-interpolated data
    range = slice_info.range * [0.999,1.001] ;padding
  endelse
  if range[1] eq range[0] then begin
    dprint, dlevel=1,'Slice at '+time_string(slice_info.trange[0])+' has no data.'
    title = 'Error: No data in range!'
    range = [1,10.]
  endif 

  
  ; Set color contour levels
  if keyword_set(logplot) then begin
    colorlevels = 10.^(indgen(levels)/float(levels)*(alog10(range[1]) - alog10(range[0])) + alog10(range[0]))
  endif else begin
    colorlevels = (indgen(levels)/float(levels)*(range[1]-range[0])+range[0])
  endelse
  
  
  ; Set colors
  thecolors = round((indgen(levels)+1)*(!d.table_size-9)/levels)+7


  ; Control X Ticks
  if size(xticks,/type) ne 0 then begin
    xticks = round(xticks)
    ;n+1 ticks actually get plotted
    nxticks = xticks gt 1 ? xticks-1:xticks 
    ;use spaces to supress labels for 0 or 1 tick
    if xticks eq 1 then xtickname = ['',' ']
    if xticks lt 1 then begin
      nxticks = 1
      xtickname = [' ',' ']
    endif
    xminor = (40 / nxticks) 
  endif
  
  
  ; Control Y ticks
  if size(yticks,/type) ne 0 then begin
    yticks = round(yticks)
    ;n+1 ticks actually get plotted
    nyticks = yticks gt 1 ? yticks-1:yticks 
    ;use spaces to supress labels for 0 or 1 tick
    if yticks eq 1 then ytickname = ['',' ']
    if yticks lt 1 then begin
      nyticks = 1
      ytickname = [' ',' ']
    endif
    yminor = (40 / nyticks) 
  endif


  ; Get Display info
  plot_part_slice2d_getinfo, slice_info, $
                             title=title, $
;                             subtitle=subtitle, $
                             xtitle=xtitle, $
                             ytitle=ytitle, $
                             ztitle=ztitle


  ; Format plot window
  if !d.name ne 'PS' then begin

    plotxcsize = plotsize / !d.x_ch_size
    plotycsize = plotsize / !d.y_ch_size
    
    xmargin = [11,15 + (aprecision-3 > 0)]
    ymargin = [4,2]
    
    tsize = strlen(title) * 1.25
  
    xcsize = (total(xmargin) + plotxcsize) > tsize
    ycsize = (total(ymargin) + plotycsize) 

    device, window_state = wins
    
    ;ensure a free window is used if possible
    wn = !d.window le 1 or !d.window ge 32  ?  $
          min( where(wins eq 0) ) > 2 : !d.window
  
    window, wn, xsize = xcsize * !d.x_ch_size, $
            ysize = ycsize * !d.y_ch_size, $
            title = title

    xmargin[0] = xmargin[0] > 0.5*(xcsize - plotxcsize)
  endif
 

  ; Plot
  contour, part_slice, xgrid, ygrid, $
      levels = colorlevels, c_color = thecolors, fill=1, $
      /isotropic, /closed, charsize=charsize, $
      title = title, $
      xmargin = xmargin, $
      ymargin = ymargin, $
      xstyle = 1, $
      ystyle = 1, $
      ticklen = 0.01,$
      xtickname = xtickname,$  ; Specify names to supress labels
      ytickname = ytickname,$  ; in cases of 0 or 1 ticks
      xminor = xminor,$    ; Allow minor ticks to persist
      yminor = yminor,$    ; when specifying # of major ticks
      xticks = nxticks,$
      yticks = nyticks,$
      xrange = xrange,$
      yrange = yrange,$
      xtitle = xtitle,$
      ytitle = ytitle



  ; Z Ticks and Custom annotations
  zticks = round(zticks)
  nzticks = zticks gt 1 ? zticks-1:zticks ;n+1 plotted 
  ztickstrings = strarr(nzticks+1)
  axis, yaxis=1, ystyle = 1+4, $
        yrange = range, ylog = logplot, $
        yticks = nzticks, $
        ytick_get = ztickvals
  for i=0, nzticks do begin
    ztickstrings[i] = formatannotation(0,0,ztickvals[i], $
      data={timeaxis:0,formatid:aprecision,scaling:0,exponent:astyle})
  endfor
  if zticks eq 1 then ztickstrings[1] = ' '
  if zticks lt 1 then begin
    nzticks = 1
    ztickstrings = [' ',' ']
  endif
  
  
  ; Draw z axis color bar
  draw_color_scale,range=range,log = logplot,yticks=nzticks, title=ztitle, $
    ytickname=ztickstrings, charsize=charsize



  ;Other Plotting Options
  ;----------------------

  ; Plot contour lines
  if keyword_set(olines) then begin

    ; set contour levels
    if keyword_set(logplot) then begin
      linelevels = 10.^(indgen(olines)/float(olines)*(alog10(range[1]) - alog10(range[0])) + alog10(range[0]))
    endif else begin
      linelevels = (indgen(olines)/float(olines)*(range[1]-range[0])+range[0])
    endelse

    contour, part_slice, xgrid, ygrid, $
        levels = linelevels, charsize=charsize,  $
        /overplot, /closed, /isotropic, $
        follow = clabels
  endif


  ; Plot axes
  if keyword_set(plotaxes) then begin
    oplot,[0.,0.],minmax(ygrid),linestyle = 2,thick = 2
    oplot,minmax(xgrid),[0.,0.],linestyle = 2,thick = 2
  endif
  

  ; Plot circle for minimum/maximum velocity 
  ; (based off energy limits) 
  if keyword_set(ecircle) then begin
    degrees = findgen(360)*!dtor
    
    ocircy=sin(degrees) * slice_info.vrange[1]
    ocircx=cos(degrees) * slice_info.vrange[1]
    icircy=sin(degrees) * slice_info.vrange[0]
    icircx=cos(degrees) * slice_info.vrange[0]
    
    
    ;adjust for subtraction of bulk velocity
    if keyword_set(slice_info.shift) then begin
      ocircy -= slice_info.shift[1]
      ocircx -= slice_info.shift[0]
      icircy -= slice_info.shift[1]
      icircx -= slice_info.shift[0]
    endif
    
    if slice_info.vrange[1] gt 0 then $
      oplot,ocircx,ocircy,thick = 2
    if slice_info.vrange[0] gt (yrange[1]-yrange[0])*0.015 then $
      oplot,icircx,icircy,thick = 2
  endif
  
  
  ; Plot the bulk velocity
  if keyword_set(plotbulk) and keyword_set(slice_info.bulk) $
    and ~keyword_set(shift) then begin
    ; bulk velocity should already be in the coords defined for
    ; the slice plane
    oplot, [0,slice_info.bulk[0]], [0,slice_info.bulk[1]], color=!d.table_size-9
  endif


  ; Plot sun direction
  if keyword_set(sundir) then begin
    
    sund = plot_part_slice2d_getsun(slice_info, fail=fail)
    
    if ~keyword_set(fail) then begin
      ;project into slice plane and normalize
      sund_p = sund[0:1]
      sund_pnorm = sund_p / sqrt(total(sund_p^2))
  
      ;plot
      oplot,[0,sund_pnorm[0]*max(xrange)],[0,sund_pnorm[1]*max(xrange)]
    endif
    
  endif


  ; Finish export:  write .png or close postscript file 
  if keyword_set(export) then begin
    if keyword_set(eps) then begin
      pclose
    endif else begin
      makepng, export, /mkdir, _extra=_extra
    endelse
  endif


;  if keyword_set(o_part_slice) then begin
;    ; apply user specified scaling
;    if keyword_set(scale) then o_part_slice *= scale
;
;    contour, o_part_slice, o_xgrid, o_ygrid, $
;             levels = thelevels, c_color = thecolors, fill=1, $
;             /isotropic, /closed, /overplot
;
;    if keyword_set(olines) then begin
;      contour, o_part_slice, o_xgrid, o_ygrid, $
;          levels = thelevels2, charsize=charsize, follow = clabels, $
;          /isotropic, /closed, /overplot
;    endif
;
;  endif

end
