
; Checks if a list of times covers a time range
;
function thm_check_tr, tr, times, margin=margin

    compile_opt idl2, hidden
  
  
  if size(margin,/type) eq 0 then begin
    margin = median(  (times - shift(times,1))[1:n_elements(times)-1]  )/2.
  endif
  
  return,  (min(times)-margin) le min(tr) && (max(times)+margin) ge max(tr) 

end


; Checks for presence of SST data within a given time range
;
function thm_check_sst_data, probe, type, trange, fail=fail

    compile_opt idl2, hidden


  r = 0
  
  times = call_function('thm_sst_'+type, probe=probe, /time)
  
  if times[0] ne 0 then begin
  
    r = thm_check_tr(trange, times)
    
  endif
  
  return, r
  
end


; Checks for presence of ESA data withing a given time range
;
function thm_check_esa_data, probe, type, trange, fail=fail

    compile_opt idl2, hidden


  r = 0

  times = call_function('get_th'+probe+'_'+type, /time)

  if size(times,/type) ne 8 && times[0] ne 0 then begin
  
    r = thm_check_tr(trange, times)
  
  endif

  return, r
  
end


;+
;NAME:
;  thm_check_part_data
;
;PURPOSE:
;  This routine checks the time ranges of the current ESA and SST  
;  data stored in the common blocks to determine if it covers a
;  particular time range.
;
;CALLING SEQUENCE:
;  bool = thm_check_part_data(probe, type, trange)
;
;INPUT:
;  probe: String specifying the probe
;  type: Standart string (or array of) specifying the type of 
;        particle data requested (e.g. 'peif', 'pseb')
;  trange: Two element array specifying the numeric time range
;
;OUTPUT:
;  1 if current data covers what is requested, 0 otherwise
;
;NOTES: 
;  
;  
;-
function thm_check_part_data, probe, type0, trange, fail=fail

    compile_opt idl2


  r = 0

  type = strlowcase(type0)
  
  valid = where( stregex(type, '^p[se][ei][rfb]$',/fold_case,/bool), nv)
  if nv gt 0 then begin
    type = type[valid]
  endif else begin
    fail = 'No valid data type(s)'
    return, 0
  endelse

  for j=0, n_elements(probe)-1 do begin
    for i=0, n_elements(type)-1 do begin
  
      if strmid(type[i],1,1) eq 'e' then begin
      
        r = thm_check_esa_data(probe[j], type[i], trange, fail=fail)
      
      endif else if strmid(type[i],1,1) eq 's' then begin
      
        r = thm_check_sst_data(probe[j], type[i], trange, fail=fail)
      
      endif
  
      if r eq 0 then return,0
  
    endfor
  endfor

  return, 1

end
